
/*
 * pyblock3: An Efficient python MPS/DMRG Library
 * Copyright (C) 2020 The pyblock3 developers. All Rights Reserved.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include "bond_info.hpp"
#include "tensor_impl.hpp"
#ifdef I
#undef I
#endif
#include <pybind11/numpy.h>
#include <tuple>
#include <vector>

namespace py = pybind11;
using namespace std;

template <typename FL>
vector<tuple<py::array_t<uint32_t>, py::array_t<uint32_t>, py::array_t<FL>,
             py::array_t<uint64_t>>>
build_mpo(py::array_t<int32_t> orb_sym, py::array_t<FL> h_values,
          py::array_t<int32_t> h_terms, double cutoff = 1E-20,
          int max_bond_dim = -1);

extern template vector<tuple<py::array_t<uint32_t>, py::array_t<uint32_t>,
                             py::array_t<double>, py::array_t<uint64_t>>>
build_mpo(py::array_t<int32_t> orb_sym, py::array_t<double> h_values,
          py::array_t<int32_t> h_terms, double cutoff = 1E-20,
          int max_bond_dim = -1);

extern template vector<
    tuple<py::array_t<uint32_t>, py::array_t<uint32_t>,
          py::array_t<complex<double>>, py::array_t<uint64_t>>>
build_mpo(py::array_t<int32_t> orb_sym, py::array_t<complex<double>> h_values,
          py::array_t<int32_t> h_terms, double cutoff = 1E-20,
          int max_bond_dim = -1);
